@@ -1,11 +1,11 @@
 /*
  * This file is part of telepathy-idle
- * 
+ *
  * Copyright (C) 2006-2007 Collabora Limited
  * Copyright (C) 2006-2007 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License 
+ * modify it under the terms of the GNU Lesser General Public License
  * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful,
@@ -18,56 +18,34 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "idle-connection.h"
+
 #include <config.h>
 
-#define DBUS_API_SUBJECT_TO_CHANGE
-
-#include <dbus/dbus-glib.h>
-#include <dbus/dbus-glib-lowlevel.h>
-
-#include <telepathy-glib/intset.h>
-#include <telepathy-glib/enums.h>
-#include <telepathy-glib/interfaces.h>
-#include <telepathy-glib/errors.h>
-#include <telepathy-glib/dbus.h>
-#include <telepathy-glib/svc-connection.h>
-#include <telepathy-glib/channel-factory-iface.h>
-
-#include <stdio.h>
-#include <stdlib.h>
-#include <time.h>
-#include <errno.h>
-
 /* strnlen */
 #define __USE_GNU
 #include <string.h>
-#include <ctype.h>
+#include <time.h>
 
-#include <unistd.h>
-#include <pthread.h>
-#include <sys/poll.h>
-#include <sys/time.h>
-#include <sys/types.h>
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
-#include <netdb.h>
+#define DBUS_API_SUBJECT_TO_CHANGE
+#include <dbus/dbus-glib.h>
 
-#include "idle-handles.h"
-#include "idle-muc-channel.h"
-
-#include "idle-ctcp.h"
-#include "idle-parser.h"
-#include "idle-server-connection.h"
-#include "idle-ssl-server-connection.h"
-#include "idle-server-connection-iface.h"
-#include "idle-im-factory.h"
-#include "idle-muc-factory.h"
-
-#include "idle-connection.h"
+#include <telepathy-glib/enums.h>
+#include <telepathy-glib/errors.h>
+#include <telepathy-glib/interfaces.h>
+#include <telepathy-glib/svc-connection.h>
+#include <telepathy-glib/channel-factory-iface.h>
 
 #define IDLE_DEBUG_FLAG IDLE_DEBUG_CONNECTION
+#include "idle-ctcp.h"
 #include "idle-debug.h"
+#include "idle-handles.h"
+#include "idle-im-factory.h"
+#include "idle-muc-factory.h"
+#include "idle-parser.h"
+#include "idle-server-connection.h"
+#include "idle-server-connection-iface.h"
+#include "idle-ssl-server-connection.h"
 
 #include "extensions/extensions.h"    /* Renaming */
 
@@ -88,6 +66,11 @@
 #define IDLE_TP_ALIAS_PAIR_TYPE (dbus_g_type_get_struct ("GValueArray", \
 			G_TYPE_UINT, G_TYPE_STRING, G_TYPE_INVALID))
 
+static void _free_alias_pair(gpointer data, gpointer user_data)
+{
+	g_boxed_free(IDLE_TP_ALIAS_PAIR_TYPE, data);
+}
+
 static void _aliasing_iface_init(gpointer, gpointer);
 static void _renaming_iface_init(gpointer, gpointer);
 
@@ -99,12 +82,17 @@ typedef struct _IdleOutputPendingMsg IdleOutputPendingMsg;
 struct _IdleOutputPendingMsg {
 	gchar *message;
 	guint priority;
+	guint64 id;
 };
 
-#define idle_output_pending_msg_new() \
-	(g_slice_new(IdleOutputPendingMsg))
-#define idle_output_pending_msg_new0() \
-	(g_slice_new0(IdleOutputPendingMsg))
+static IdleOutputPendingMsg *idle_output_pending_msg_new() {
+	IdleOutputPendingMsg *msg = g_slice_new(IdleOutputPendingMsg);
+	static guint64 last_id = 0;
+
+	msg->id = last_id++;
+
+	return msg;
+}
 
 static void idle_output_pending_msg_free(IdleOutputPendingMsg *msg) {
 	if (!msg)
@@ -117,7 +105,13 @@ static void idle_output_pending_msg_free(IdleOutputPendingMsg *msg) {
 static gint pending_msg_compare(gconstpointer a, gconstpointer b, gpointer unused) {
 	const IdleOutputPendingMsg *msg1 = a, *msg2 = b;
 
-	return (msg1->priority > msg2->priority) ? -1 : 1;
+	if (msg1->priority == msg2->priority) {
+		/* prefer the message with the lower id */
+		return (msg1->id - msg2->id);
+	}
+
+	/* prefer the message with the higher priority */
+	return (msg2->priority - msg1->priority);
 }
 
 enum {
@@ -152,6 +146,12 @@ struct _IdleConnectionPrivate {
 	char *quit_message;
 	gboolean use_ssl;
 
+	/* the string used by the a server as a prefix to any messages we send that
+	 * it relays to other users.  We need to know this so we can keep our sent
+	 * messages short enough that they still fit in the 512-byte limit even with
+	 * this prefix added */
+	char *relay_prefix;
+
 	/* output message queue */
 	GQueue *msg_queue;
 
@@ -163,6 +163,7 @@ struct _IdleConnectionPrivate {
 
 	/* if we are quitting asynchronously */
 	gboolean quitting;
+	guint force_disconnect_id;
 
 	/* AliasChanged aggregation */
 	GPtrArray *queued_aliases;
@@ -172,7 +173,7 @@ struct _IdleConnectionPrivate {
 	gboolean dispose_has_run;
 };
 
-#define IDLE_CONNECTION_GET_PRIVATE(o)     (G_TYPE_INSTANCE_GET_PRIVATE ((o), IDLE_TYPE_CONNECTION, IdleConnectionPrivate))
+#define IDLE_CONNECTION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), IDLE_TYPE_CONNECTION, IdleConnectionPrivate))
 
 static void _iface_create_handle_repos(TpBaseConnection *self, TpHandleRepoIface **repos);
 static GPtrArray *_iface_create_channel_factories(TpBaseConnection *self);
@@ -187,6 +188,7 @@ static IdleParserHandlerResult _nickname_in_use_handler(IdleParser *parser, Idle
 static IdleParserHandlerResult _ping_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data);
 static IdleParserHandlerResult _version_privmsg_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data);
 static IdleParserHandlerResult _welcome_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data);
+static IdleParserHandlerResult _whois_user_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data);
 
 static void sconn_status_changed_cb(IdleServerConnectionIface *sconn, IdleServerConnectionState state, IdleServerConnectionStateReason reason, IdleConnection *conn);
 static void sconn_received_cb(IdleServerConnectionIface *sconn, gchar *raw_msg, IdleConnection *conn);
@@ -195,10 +197,11 @@ static void irc_handshakes(IdleConnection *conn);
 static void send_quit_request(IdleConnection *conn);
 static void connection_connect_cb(IdleConnection *conn, gboolean success, TpConnectionStatusReason fail_reason);
 static void connection_disconnect_cb(IdleConnection *conn, TpConnectionStatusReason reason);
-static void send_irc_cmd(IdleConnection *conn, const gchar *msg);
+static gboolean idle_connection_hton(IdleConnection *obj, const gchar *input, gchar **output, GError **_error);
+static void idle_connection_ntoh(IdleConnection *obj, const gchar *input, gchar **output);
 
-static void idle_connection_init (IdleConnection *obj) {
-	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE (obj);
+static void idle_connection_init(IdleConnection *obj) {
+	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(obj);
 
 	priv->sconn_status = SERVER_CONNECTION_STATE_NOT_CONNECTED;
 	priv->msg_queue = g_queue_new();
@@ -220,32 +223,40 @@ static void idle_connection_set_property(GObject *obj, guint prop_id, const GVal
 			g_free(priv->nickname);
 			priv->nickname = g_value_dup_string(value);
 			break;
+
 		case PROP_SERVER:
 			g_free(priv->server);
 			priv->server = g_value_dup_string(value);
 			break;
+
 		case PROP_PORT:
 			priv->port = g_value_get_uint(value);
 			break;
+
 		case PROP_PASSWORD:
 			g_free(priv->password);
 			priv->password = g_value_dup_string(value);
 			break;
+
 		case PROP_REALNAME:
 			g_free(priv->realname);
 			priv->realname = g_value_dup_string(value);
 			break;
+
 		case PROP_CHARSET:
 			g_free(priv->charset);
 			priv->charset = g_value_dup_string(value);
 			break;
+
 		case PROP_QUITMESSAGE:
 			g_free(priv->quit_message);
 			priv->quit_message = g_value_dup_string(value);
 			break;
+
 		case PROP_USE_SSL:
 			priv->use_ssl = g_value_get_boolean(value);
 			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, prop_id, pspec);
 			break;
@@ -259,27 +270,35 @@ static void idle_connection_get_property(GObject *obj, guint prop_id, GValue *va
 		case PROP_NICKNAME:
 			g_value_set_string(value, priv->nickname);
 			break;
+
 		case PROP_SERVER:
 			g_value_set_string(value, priv->server);
 			break;
+
 		case PROP_PORT:
 			g_value_set_uint(value, priv->port);
 			break;
+
 		case PROP_PASSWORD:
 			g_value_set_string(value, priv->password);
 			break;
+
 		case PROP_REALNAME:
 			g_value_set_string(value, priv->realname);
 			break;
+
 		case PROP_CHARSET:
 			g_value_set_string(value, priv->charset);
 			break;
+
 		case PROP_QUITMESSAGE:
 			g_value_set_string(value, priv->quit_message);
 			break;
+
 		case PROP_USE_SSL:
 			g_value_set_boolean(value, priv->use_ssl);
 			break;
+
 		default:
 			G_OBJECT_WARN_INVALID_PROPERTY_ID(obj, prop_id, pspec);
 			break;
@@ -311,8 +330,8 @@ static void idle_connection_dispose (GObject *object) {
 
 	g_object_unref(self->parser);
 
-	if (G_OBJECT_CLASS (idle_connection_parent_class)->dispose)
-		G_OBJECT_CLASS (idle_connection_parent_class)->dispose (object);
+	if (G_OBJECT_CLASS(idle_connection_parent_class)->dispose)
+		G_OBJECT_CLASS(idle_connection_parent_class)->dispose (object);
 }
 
 static void idle_connection_finalize (GObject *object) {
@@ -324,6 +343,7 @@ static void idle_connection_finalize (GObject *object) {
 	g_free(priv->password);
 	g_free(priv->realname);
 	g_free(priv->charset);
+	g_free(priv->relay_prefix);
 	g_free(priv->quit_message);
 
 	while ((msg = g_queue_pop_head(priv->msg_queue)) != NULL)
@@ -331,10 +351,10 @@ static void idle_connection_finalize (GObject *object) {
 
 	g_queue_free(priv->msg_queue);
 
-	G_OBJECT_CLASS (idle_connection_parent_class)->finalize(object);
+	G_OBJECT_CLASS(idle_connection_parent_class)->finalize(object);
 }
 
-static void idle_connection_class_init (IdleConnectionClass *klass) {
+static void idle_connection_class_init(IdleConnectionClass *klass) {
 	GObjectClass *object_class = G_OBJECT_CLASS(klass);
 	TpBaseConnectionClass *parent_class = TP_BASE_CONNECTION_CLASS(klass);
 	GParamSpec *param_spec;
@@ -353,13 +373,13 @@ static void idle_connection_class_init (IdleConnectionClass *klass) {
 
 	parent_class->create_handle_repos = _iface_create_handle_repos;
 	parent_class->get_unique_connection_name = _iface_get_unique_connection_name;
-  parent_class->create_channel_factories = _iface_create_channel_factories;
-  parent_class->connecting = NULL;
+	parent_class->create_channel_factories = _iface_create_channel_factories;
+	parent_class->connecting = NULL;
 	parent_class->connected = NULL;
-  parent_class->disconnected = _iface_disconnected;
-  parent_class->shut_down = _iface_shut_down;
-  parent_class->start_connecting = _iface_start_connecting;
-  parent_class->interfaces_always_present = interfaces_always_present;
+	parent_class->disconnected = _iface_disconnected;
+	parent_class->shut_down = _iface_shut_down;
+	parent_class->start_connecting = _iface_start_connecting;
+	parent_class->interfaces_always_present = interfaces_always_present;
 
 	param_spec = g_param_spec_string("nickname", "IRC nickname", "The nickname to be visible to others in IRC.", NULL, G_PARAM_READWRITE | G_PARAM_STATIC_NAME | G_PARAM_STATIC_BLURB);
 	g_object_class_install_property(object_class, PROP_NICKNAME, param_spec);
@@ -414,12 +434,25 @@ static gchar *_iface_get_unique_connection_name(TpBaseConnection *self) {
 
 static gboolean _finish_shutdown_idle_func(gpointer data) {
 	TpBaseConnection *conn = TP_BASE_CONNECTION(data);
+	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
+	if (priv->force_disconnect_id != 0) {
+		g_source_remove(priv->force_disconnect_id);
+	}
 
 	tp_base_connection_finish_shutdown(conn);
 
 	return FALSE;
 }
 
+static gboolean
+_force_disconnect (gpointer data)
+{
+	IdleConnection *conn = IDLE_CONNECTION(data);
+	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
+	idle_server_connection_iface_disconnect(priv->conn, NULL);
+	return FALSE;
+}
+
 static void _iface_disconnected(TpBaseConnection *self) {
 	IdleConnection *conn = IDLE_CONNECTION(self);
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
@@ -427,11 +460,16 @@ static void _iface_disconnected(TpBaseConnection *self) {
 	send_quit_request(conn);
 
 	priv->quitting = TRUE;
+	/* don't handle any more messages, we're quitting.  See e.g. bug #19762 */
+	idle_parser_remove_handlers_by_data(conn->parser, conn);
+	/* schedule forceful disconnect for 2 seconds if the remote server doesn't
+	 * respond or disconnect before then */
+	priv->force_disconnect_id = g_timeout_add_seconds(2, _force_disconnect, conn);
 }
 
 static void _iface_shut_down(TpBaseConnection *self) {
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(self);
-	GError *error;
+	GError *error = NULL;
 
 	if (priv->quitting)
 		return;
@@ -489,6 +527,7 @@ static gboolean _iface_start_connecting(TpBaseConnection *self, GError **error)
 		idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_ERRONEOUSNICKNAME, _erroneous_nickname_handler, conn);
 		idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_NICKNAMEINUSE, _nickname_in_use_handler, conn);
 		idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_WELCOME, _welcome_handler, conn);
+		idle_parser_add_handler(conn->parser, IDLE_PARSER_NUMERIC_WHOISUSER, _whois_user_handler, conn);
 
 		idle_parser_add_handler(conn->parser, IDLE_PARSER_CMD_PING, _ping_handler, conn);
 
@@ -496,7 +535,7 @@ static gboolean _iface_start_connecting(TpBaseConnection *self, GError **error)
 		idle_parser_add_handler(conn->parser, IDLE_PARSER_PREFIXCMD_PRIVMSG_USER, _version_privmsg_handler, conn);
 
 		irc_handshakes(conn);
-	}	else {
+	} else {
 		IDLE_DEBUG("conn already open!");
 
 		g_set_error(error, TP_ERRORS, TP_ERROR_NOT_AVAILABLE, "connection already open!");
@@ -513,15 +552,24 @@ static void sconn_status_changed_cb(IdleServerConnectionIface *sconn, IdleServer
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
 	TpConnectionStatusReason tp_reason;
 
+	/* cancel scheduled forced disconnect since we are now disconnected */
+	if (state == SERVER_CONNECTION_STATE_NOT_CONNECTED &&
+		priv->force_disconnect_id) {
+		g_source_remove(priv->force_disconnect_id);
+		priv->force_disconnect_id = 0;
+	}
+
 	IDLE_DEBUG("called with state %u", state);
 
 	switch (reason) {
 		case SERVER_CONNECTION_STATE_REASON_ERROR:
 			tp_reason = TP_CONNECTION_STATUS_REASON_NETWORK_ERROR;
 			break;
+
 		case SERVER_CONNECTION_STATE_REASON_REQUESTED:
 			tp_reason = TP_CONNECTION_STATUS_REASON_REQUESTED;
 			break;
+
 		default:
 			g_assert_not_reached();
 			break;
@@ -540,8 +588,10 @@ static void sconn_status_changed_cb(IdleServerConnectionIface *sconn, IdleServer
 				connection_disconnect_cb(conn, tp_reason);
 			}
 			break;
+
 		case SERVER_CONNECTION_STATE_CONNECTING:
 			break;
+
 		case SERVER_CONNECTION_STATE_CONNECTED:
 			if ((priv->msg_queue_timeout == 0) && (g_queue_get_length(priv->msg_queue) > 0)) {
 				IDLE_DEBUG("we had messages in queue, start unloading them now");
@@ -549,6 +599,7 @@ static void sconn_status_changed_cb(IdleServerConnectionIface *sconn, IdleServer
 				priv->msg_queue_timeout = g_timeout_add(MSG_QUEUE_TIMEOUT, msg_queue_timeout_cb, conn);
 			}
 			break;
+
 		default:
 			g_assert_not_reached();
 			break;
@@ -566,19 +617,13 @@ static void sconn_received_cb(IdleServerConnectionIface *sconn, gchar *raw_msg,
 	g_free(converted);
 }
 
-gboolean _idle_connection_send(IdleConnection *conn, const gchar *msg) {
-	send_irc_cmd(conn, msg);
-
-	return TRUE;
-}
-
 static gboolean msg_queue_timeout_cb(gpointer user_data) {
 	IdleConnection *conn = IDLE_CONNECTION(user_data);
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
 	int i, j;
 	IdleOutputPendingMsg *output_msg;
-	gchar msg[IRC_MSG_MAXLEN+3];
-	GError *error;
+	gchar msg[IRC_MSG_MAXLEN + 3];
+	GError *error = NULL;
 
 	IDLE_DEBUG("called");
 
@@ -597,12 +642,12 @@ static gboolean msg_queue_timeout_cb(gpointer user_data) {
 		return FALSE;
 	}
 
-	g_strlcpy(msg, output_msg->message, IRC_MSG_MAXLEN+3);
+	g_strlcpy(msg, output_msg->message, IRC_MSG_MAXLEN + 3);
 
 	for (i = 1; i < MSG_QUEUE_UNLOAD_AT_A_TIME; i++) {
 		output_msg = g_queue_peek_nth(priv->msg_queue, i);
 
-		if ((output_msg != NULL) && ((strlen(msg) + strlen(output_msg->message)) < IRC_MSG_MAXLEN+2))
+		if ((output_msg != NULL) && ((strlen(msg) + strlen(output_msg->message)) < IRC_MSG_MAXLEN + 2))
 			strcat(msg, output_msg->message);
 		else
 			break;
@@ -615,11 +660,10 @@ static gboolean msg_queue_timeout_cb(gpointer user_data) {
 		}
 
 		priv->last_msg_sent = time(NULL);
-	}
-	else {
+	} else {
 		IDLE_DEBUG("low-level network connection failed to send: %s", error->message);
 
-		g_error_free(error);		
+		g_error_free(error);
 	}
 
 	return TRUE;
@@ -628,13 +672,13 @@ static gboolean msg_queue_timeout_cb(gpointer user_data) {
 /**
  * Queue a IRC command for sending, clipping it to IRC_MSG_MAXLEN bytes and appending the required <CR><LF> to it
  */
-static void send_irc_cmd_full(IdleConnection *conn, const gchar *msg, guint priority) {
-	gchar cmd[IRC_MSG_MAXLEN+3];
+static void _send_with_priority(IdleConnection *conn, const gchar *msg, guint priority) {
+	gchar cmd[IRC_MSG_MAXLEN + 3];
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
 	int len;
-	GError *error;
+	GError *error = NULL;
 	gchar *converted;
-	GError *convert_error;
+	GError *convert_error = NULL;
 	time_t curr_time = time(NULL);
 	IdleOutputPendingMsg *output_msg;
 
@@ -642,7 +686,7 @@ static void send_irc_cmd_full(IdleConnection *conn, const gchar *msg, guint prio
 
 	/* Clip the message */
 
-	g_strlcpy(cmd, msg, IRC_MSG_MAXLEN+1);
+	g_strlcpy(cmd, msg, IRC_MSG_MAXLEN + 1);
 
 	/* Append <CR><LF> */
 
@@ -664,7 +708,7 @@ static void send_irc_cmd_full(IdleConnection *conn, const gchar *msg, guint prio
 		if (!idle_server_connection_iface_send(priv->conn, converted, &error)) {
 			IDLE_DEBUG("server connection failed to send: %s", error->message);
 			g_error_free(error);
-		}	else {
+		} else {
 			g_free(converted);
 			return;
 		}
@@ -677,11 +721,31 @@ static void send_irc_cmd_full(IdleConnection *conn, const gchar *msg, guint prio
 	g_queue_insert_sorted(priv->msg_queue, output_msg, pending_msg_compare, NULL);
 
 	if (!priv->msg_queue_timeout)
-		priv->msg_queue_timeout = g_timeout_add(MSG_QUEUE_TIMEOUT*1024, msg_queue_timeout_cb, conn);
+		priv->msg_queue_timeout = g_timeout_add(MSG_QUEUE_TIMEOUT * 1000, msg_queue_timeout_cb, conn);
 }
 
-static void send_irc_cmd(IdleConnection *conn, const gchar *msg) {
-	return send_irc_cmd_full(conn, msg, SERVER_CMD_NORMAL_PRIORITY);	
+void idle_connection_send(IdleConnection *conn, const gchar *msg) {
+	return _send_with_priority(conn, msg, SERVER_CMD_NORMAL_PRIORITY);
+}
+
+gsize
+idle_connection_get_max_message_length(IdleConnection *conn)
+{
+	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
+	if (priv->relay_prefix != NULL) {
+		/* server will add ':<relay_prefix> ' to all messages it relays on to
+		 * other users.  the +2 is for the initial : and the trailing space */
+		return IRC_MSG_MAXLEN - (strlen(priv->relay_prefix) + 2);
+	}
+	/* Before we've gotten our user info, we don't know how long our relay
+	 * prefix will be, so just assume worst-case.  The max possible prefix is:
+	 * ':<15 char nick>!<? char username>@<63 char hostname> ' == 1 + 15 + 1 + ?
+	 * + 1 + 63 + 1 == 82 + ?
+	 * I haven't been able to find a definitive reference for the max username
+	 * length, but the testing I've done seems to indicate that 8-10 is a
+	 * common limit.  I'll add some extra buffer to be safe.
+	 * */
+	return IRC_MSG_MAXLEN - 100;
 }
 
 static IdleParserHandlerResult _erroneous_nickname_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
@@ -702,11 +766,7 @@ static IdleParserHandlerResult _nick_handler(IdleParser *parser, IdleParserMessa
 		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 
 	if (old_handle == conn->parent.self_handle) {
-		TpHandleRepoIface *handles = tp_base_connection_get_handles(TP_BASE_CONNECTION(conn), TP_HANDLE_TYPE_CONTACT);
-
-		tp_handle_unref(handles, conn->parent.self_handle);
-		conn->parent.self_handle = new_handle;
-		tp_handle_ref(handles, new_handle);
+		tp_base_connection_set_self_handle(TP_BASE_CONNECTION(conn), new_handle);
 	}
 
 	idle_svc_connection_interface_renaming_emit_renamed(IDLE_SVC_CONNECTION_INTERFACE_RENAMING(conn), old_handle, new_handle);
@@ -729,7 +789,7 @@ static IdleParserHandlerResult _ping_handler(IdleParser *parser, IdleParserMessa
 	IdleConnection *conn = IDLE_CONNECTION(user_data);
 
 	gchar *reply = g_strdup_printf("PONG %s", g_value_get_string(g_value_array_get_nth(args, 0)));
-	send_irc_cmd_full (conn, reply, SERVER_CMD_MAX_PRIORITY);
+	_send_with_priority(conn, reply, SERVER_CMD_MAX_PRIORITY);
 	g_free(reply);
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
@@ -756,20 +816,40 @@ static IdleParserHandlerResult _version_privmsg_handler(IdleParser *parser, Idle
 static IdleParserHandlerResult _welcome_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
 	IdleConnection *conn = IDLE_CONNECTION(user_data);
 	TpHandle handle = g_value_get_uint(g_value_array_get_nth(args, 0));
-	TpHandleRepoIface *handles = tp_base_connection_get_handles(TP_BASE_CONNECTION(conn), TP_HANDLE_TYPE_CONTACT);
-	
-	conn->parent.self_handle = handle;
-	tp_handle_ref(handles, handle);
-	g_assert(tp_handle_is_valid(handles, handle, NULL));
+
+	tp_base_connection_set_self_handle(TP_BASE_CONNECTION(conn), handle);
 
 	connection_connect_cb(conn, TRUE, 0);
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
 }
 
+static IdleParserHandlerResult
+_whois_user_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data)
+{
+	IdleConnection *conn = IDLE_CONNECTION(user_data);
+	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
+
+	/* message format: <nick> <user> <host> * :<real name> */
+	TpHandle handle = g_value_get_uint(g_value_array_get_nth(args, 0));
+	TpHandle self = tp_base_connection_get_self_handle(TP_BASE_CONNECTION(conn));
+	if (handle == self) {
+			if (priv->relay_prefix != NULL) {
+					g_free(priv->relay_prefix);
+			}
+			const char* user = g_value_get_string(g_value_array_get_nth(args, 1));
+			const char* host = g_value_get_string(g_value_array_get_nth(args, 2));
+			priv->relay_prefix = g_strdup_printf("%s!%s@%s", priv->nickname, user, host);
+			IDLE_DEBUG("user host prefix = %s", priv->relay_prefix);
+
+			return IDLE_PARSER_HANDLER_RESULT_HANDLED;
+	}
+	return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
+}
+
 static void irc_handshakes(IdleConnection *conn) {
 	IdleConnectionPrivate *priv;
-	gchar msg[IRC_MSG_MAXLEN+1];
+	gchar msg[IRC_MSG_MAXLEN + 1];
 
 	g_assert(conn != NULL);
 	g_assert(IDLE_IS_CONNECTION(conn));
@@ -777,24 +857,28 @@ static void irc_handshakes(IdleConnection *conn) {
 	priv = IDLE_CONNECTION_GET_PRIVATE(conn);
 
 	if ((priv->password != NULL) && (priv->password[0] != '\0')) {
-		g_snprintf(msg, IRC_MSG_MAXLEN+1, "PASS %s", priv->password);
-		send_irc_cmd_full(conn, msg, SERVER_CMD_NORMAL_PRIORITY + 1);
+		g_snprintf(msg, IRC_MSG_MAXLEN + 1, "PASS %s", priv->password);
+		_send_with_priority(conn, msg, SERVER_CMD_NORMAL_PRIORITY + 1);
 	}
 
-	g_snprintf(msg, IRC_MSG_MAXLEN+1, "NICK %s", priv->nickname);
-	send_irc_cmd(conn, msg);
+	g_snprintf(msg, IRC_MSG_MAXLEN + 1, "NICK %s", priv->nickname);
+	idle_connection_send(conn, msg);
 
-	g_snprintf(msg, IRC_MSG_MAXLEN+1, "USER %s %u * :%s", priv->nickname, 8, priv->realname);
-	send_irc_cmd(conn, msg);
+	g_snprintf(msg, IRC_MSG_MAXLEN + 1, "USER %s %u * :%s", priv->nickname, 8, priv->realname);
+	idle_connection_send(conn, msg);
+
+	/* gather some information about ourselves */
+	g_snprintf(msg, IRC_MSG_MAXLEN + 1, "WHOIS %s", priv->nickname);
+	idle_connection_send(conn, msg);
 }
 
 static void send_quit_request(IdleConnection *conn) {
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(conn);
-	gchar cmd[IRC_MSG_MAXLEN+1];
+	gchar cmd[IRC_MSG_MAXLEN + 1];
 
-	g_snprintf(cmd, IRC_MSG_MAXLEN+1, "QUIT :%s", priv->quit_message);
+	g_snprintf(cmd, IRC_MSG_MAXLEN + 1, "QUIT :%s", priv->quit_message);
 
-	send_irc_cmd_full(conn, cmd, SERVER_CMD_MAX_PRIORITY);
+	_send_with_priority(conn, cmd, SERVER_CMD_MAX_PRIORITY);
 }
 
 static void connection_connect_cb(IdleConnection *conn, gboolean success, TpConnectionStatusReason fail_reason) {
@@ -879,6 +963,7 @@ void idle_connection_emit_queued_aliases_changed(IdleConnection *conn) {
 
 	tp_svc_connection_interface_aliasing_emit_aliases_changed(conn, priv->queued_aliases);
 
+	g_ptr_array_foreach(priv->queued_aliases, _free_alias_pair, NULL);
 	g_ptr_array_free(priv->queued_aliases, TRUE);
 	priv->queued_aliases = NULL;
 
@@ -901,7 +986,7 @@ static void idle_connection_request_aliases(TpSvcConnectionInterfaceAliasing *if
 	}
 
 	const gchar **aliases = g_new0(const gchar *, handles->len + 1);
-	for (int i = 0; i < handles->len; i++) {
+	for (guint i = 0; i < handles->len; i++) {
 		TpHandle handle = g_array_index(handles, TpHandle, i);
 
 		const gchar *alias = tp_handle_get_qdata(repo, handle, _canon_nick_quark());
@@ -933,7 +1018,7 @@ static gboolean _send_rename_request(IdleConnection *obj, const gchar *nick, DBu
 
 	gchar msg[IRC_MSG_MAXLEN + 1];
 	g_snprintf(msg, IRC_MSG_MAXLEN + 1, "NICK %s", nick);
-	send_irc_cmd(obj, msg);
+	idle_connection_send(obj, msg);
 
 	return TRUE;
 }
@@ -960,14 +1045,13 @@ static void idle_connection_set_aliases(TpSvcConnectionInterfaceAliasing *iface,
 		tp_svc_connection_interface_aliasing_return_from_set_aliases(context);
 }
 
-gboolean idle_connection_hton(IdleConnection *obj, const gchar *input, gchar **output, GError **_error) {
+static gboolean idle_connection_hton(IdleConnection *obj, const gchar *input, gchar **output, GError **_error) {
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(obj);
 	GError *error = NULL;
 	gsize bytes_written;
 	gchar *ret;
 
-	if (input == NULL)
-	{
+	if (input == NULL) {
 		*output = NULL;
 		return TRUE;
 	}
@@ -986,7 +1070,7 @@ gboolean idle_connection_hton(IdleConnection *obj, const gchar *input, gchar **o
 	return TRUE;
 }
 
-void idle_connection_ntoh(IdleConnection *obj, const gchar *input, gchar **output) {
+static void idle_connection_ntoh(IdleConnection *obj, const gchar *input, gchar **output) {
 	IdleConnectionPrivate *priv = IDLE_CONNECTION_GET_PRIVATE(obj);
 	GError *error = NULL;
 	gsize bytes_written;
@@ -1007,7 +1091,7 @@ void idle_connection_ntoh(IdleConnection *obj, const gchar *input, gchar **outpu
 		ret = g_strdup(input);
 
 		for (p = ret; *p != '\0'; p++) {
-			if (*p & (1<<7))
+			if (*p & (1 << 7))
 				*p = '?';
 		}
 	}
