@@ -1,11 +1,11 @@
 /*
  * This file is part of telepathy-idle
- * 
+ *
  * Copyright (C) 2006-2007 Collabora Limited
  * Copyright (C) 2006-2007 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License 
+ * modify it under the terms of the GNU Lesser General Public License
  * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful,
@@ -22,20 +22,18 @@
 #define __IDLE_CONNECTION_H__
 
 #include <glib-object.h>
-#include <dbus/dbus-glib.h>
-
 #include <telepathy-glib/base-connection.h>
+#include <telepathy-glib/handle.h>
 
-typedef struct _IdleConnection IdleConnection;
-typedef struct _IdleConnectionClass IdleConnectionClass;
-
-#include "idle-handles.h"
 #include "idle-parser.h"
 
 #define IRC_MSG_MAXLEN 510
 
 G_BEGIN_DECLS
 
+typedef struct _IdleConnection IdleConnection;
+typedef struct _IdleConnectionClass IdleConnectionClass;
+
 struct _IdleConnectionClass {
 	TpBaseConnectionClass parent_class;
 };
@@ -49,25 +47,22 @@ GType idle_connection_get_type(void);
 
 /* TYPE MACROS */
 #define IDLE_TYPE_CONNECTION \
-  (idle_connection_get_type())
+	(idle_connection_get_type())
 #define IDLE_CONNECTION(obj) \
-  (G_TYPE_CHECK_INSTANCE_CAST((obj), IDLE_TYPE_CONNECTION, IdleConnection))
+	(G_TYPE_CHECK_INSTANCE_CAST((obj), IDLE_TYPE_CONNECTION, IdleConnection))
 #define IDLE_CONNECTION_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_CAST((klass), IDLE_TYPE_CONNECTION, IdleConnectionClass))
+	(G_TYPE_CHECK_CLASS_CAST((klass), IDLE_TYPE_CONNECTION, IdleConnectionClass))
 #define IDLE_IS_CONNECTION(obj) \
-  (G_TYPE_CHECK_INSTANCE_TYPE((obj), IDLE_TYPE_CONNECTION))
+	(G_TYPE_CHECK_INSTANCE_TYPE((obj), IDLE_TYPE_CONNECTION))
 #define IDLE_IS_CONNECTION_CLASS(klass) \
-  (G_TYPE_CHECK_CLASS_TYPE((klass), IDLE_TYPE_CONNECTION))
+	(G_TYPE_CHECK_CLASS_TYPE((klass), IDLE_TYPE_CONNECTION))
 #define IDLE_CONNECTION_GET_CLASS(obj) \
-  (G_TYPE_INSTANCE_GET_CLASS ((obj), IDLE_TYPE_CONNECTION, IdleConnectionClass))
-
-gboolean _idle_connection_send(IdleConnection *conn, const gchar *msg);
+	(G_TYPE_INSTANCE_GET_CLASS((obj), IDLE_TYPE_CONNECTION, IdleConnectionClass))
 
 void idle_connection_canon_nick_receive(IdleConnection *conn, TpHandle handle, const gchar *canon_nick);
 void idle_connection_emit_queued_aliases_changed(IdleConnection *conn);
-
-gboolean idle_connection_hton(IdleConnection *obj, const gchar *input, gchar **output, GError **error);
-void idle_connection_ntoh(IdleConnection *obj, const gchar *input, gchar **output);
+void idle_connection_send(IdleConnection *conn, const gchar *msg);
+gsize idle_connection_get_max_message_length(IdleConnection *conn);
 
 G_END_DECLS
 
