@@ -1,11 +1,11 @@
 /*
  * This file is part of telepathy-idle
- * 
+ *
  * Copyright (C) 2006-2007 Collabora Limited
  * Copyright (C) 2006-2007 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License 
+ * modify it under the terms of the GNU Lesser General Public License
  * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful,
@@ -18,29 +18,23 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
+#include "idle-handles.h"
+
 #include <glib.h>
-#include <string.h>
 #include <ctype.h>
+#include <string.h>
 
 #include <telepathy-glib/errors.h>
 #include <telepathy-glib/handle-repo-dynamic.h>
 
-#include "idle-handles.h"
-
-#include "idle-connection.h"
-
-static gboolean _nickname_is_valid(const gchar *nickname) {
-	gsize len;
-	gunichar ucs4char;
+gboolean idle_nickname_is_valid(const gchar *nickname) {
 	const gchar *char_pos = nickname;
 
-	len = g_utf8_strlen(nickname, -1);
-
-	if (!len)
+	/* FIXME: also check for max length? */
+	if (!nickname || *nickname == '\0')
 		return FALSE;
 
-	while (char_pos != NULL) {
-		ucs4char = g_utf8_get_char_validated(char_pos, -1);
+	while (TRUE) {
 
 		switch (*char_pos) {
 			case '[':
@@ -52,25 +46,32 @@ static gboolean _nickname_is_valid(const gchar *nickname) {
 			case '{':
 			case '|':
 			case '}':
-			case '-':
 				break;
+
+			/* - not allowed as first char in a nickname */
+			case '-':
+				if (char_pos == nickname)
+					return FALSE;
+				break;
+
 			case '\0':
 				return TRUE;
 				break;
+
 			default:
-				if (!(g_unichar_isalpha(ucs4char) || ((char_pos != nickname) && g_unichar_isdigit(ucs4char))))
+				if (!(g_ascii_isalpha(*char_pos) || ((char_pos != nickname) && g_ascii_isdigit(*char_pos))))
 					return FALSE;
 				break;
 		}
 
-		char_pos = g_utf8_find_next_char(char_pos, NULL);
+		++char_pos;
 	}
 
 	return TRUE;
 }
 
 static gboolean _channelname_is_valid(const gchar *channel) {
-	const static gchar not_allowed_chars[] = {' ', '\007', ',', '\r', '\n', ':', '\0'};
+	static const gchar not_allowed_chars[] = {' ', '\007', ',', '\r', '\n', ':', '\0'};
 
 	if ((channel[0] != '#') && (channel[0] != '!') && (channel[0] != '&') && (channel[0] != '+'))
 		return FALSE;
@@ -86,7 +87,7 @@ static gboolean _channelname_is_valid(const gchar *channel) {
 		}
 	}
 
-	const gchar *tmp = strchr(channel+1, ':');
+	const gchar *tmp = strchr(channel + 1, ':');
 	if (tmp != NULL) {
 		for (const gchar *tmp2 = channel + 1; tmp2 != tmp; tmp2++) {
 			if (strchr(not_allowed_chars, *tmp2))
@@ -97,8 +98,8 @@ static gboolean _channelname_is_valid(const gchar *channel) {
 			if (strchr(not_allowed_chars, *tmp2))
 				return FALSE;
 		}
-	}	else {
-		for (const gchar *tmp2 = channel+1; tmp2 != channel + len; tmp2++) {
+	} else {
+		for (const gchar *tmp2 = channel + 1; tmp2 != channel + len; tmp2++) {
 			if (strchr(not_allowed_chars, *tmp2))
 				return FALSE;
 		}
@@ -108,7 +109,7 @@ static gboolean _channelname_is_valid(const gchar *channel) {
 }
 
 static gchar *_nick_normalize_func(TpHandleRepoIface *repo, const gchar *id, gpointer ctx, GError **error) {
-	if (!_nickname_is_valid(id)) {
+	if (!idle_nickname_is_valid(id)) {
 		g_set_error(error, TP_ERRORS, TP_ERROR_INVALID_HANDLE, "invalid nickname");
 		return NULL;
 	}
@@ -121,12 +122,6 @@ static gchar *_nick_normalize_func(TpHandleRepoIface *repo, const gchar *id, gpo
 static gchar *_channel_normalize_func(TpHandleRepoIface *repo, const gchar *id, gpointer ctx, GError **error) {
 	gchar *channel = g_strdup(id);
 
-	if ((channel[0] != '#') && (channel[0] != '!') && (channel[0] != '&') && (channel[0] != '+')) {
-		gchar *tmp = channel;
-		channel = g_strdup_printf("#%s", channel);
-		g_free(tmp);
-	}
-
 	if (!_channelname_is_valid(channel)) {
 		g_set_error(error, TP_ERRORS, TP_ERROR_INVALID_HANDLE, "invalid channel ID");
 		return NULL;
