@@ -1,11 +1,11 @@
 /*
  * This file is part of telepathy-idle
- * 
+ *
  * Copyright (C) 2006-2007 Collabora Limited
  * Copyright (C) 2006-2007 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License 
+ * modify it under the terms of the GNU Lesser General Public License
  * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful,
@@ -22,15 +22,12 @@
 #define __IDLE_HANDLES_H__
 
 #include <glib.h>
-
-#include <telepathy-glib/enums.h>
 #include <telepathy-glib/handle-repo.h>
 
-#include "idle-connection.h"
-
 G_BEGIN_DECLS
 
 void idle_handle_repos_init(TpHandleRepoIface **handles);
+gboolean idle_nickname_is_valid(const gchar *nickname);
 
 G_END_DECLS
 
