@@ -1,11 +1,11 @@
 /*
  * This file is part of telepathy-idle
- * 
+ *
  * Copyright (C) 2006-2007 Collabora Limited
  * Copyright (C) 2006-2007 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License 
+ * modify it under the terms of the GNU Lesser General Public License
  * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful,
@@ -20,25 +20,23 @@
 
 #include "idle-muc-factory.h"
 
-#include "idle-connection.h"
-#include "idle-ctcp.h"
-#include "idle-parser.h"
-#include "idle-text.h"
-
 #include <time.h>
 
-#include <glib.h>
-
 #include <telepathy-glib/channel-factory-iface.h>
 #include <telepathy-glib/interfaces.h>
 
 #define IDLE_DEBUG_FLAG IDLE_DEBUG_MUC
+#include "idle-connection.h"
+#include "idle-ctcp.h"
 #include "idle-debug.h"
+#include "idle-muc-channel.h"
+#include "idle-parser.h"
+#include "idle-text.h"
 
-static void _channel_factory_iface_init(gpointer g_iface, gpointer iface_data);
+static void _factory_iface_init(gpointer, gpointer);
 
 G_DEFINE_TYPE_WITH_CODE(IdleMUCFactory, idle_muc_factory, G_TYPE_OBJECT,
-		G_IMPLEMENT_INTERFACE(TP_TYPE_CHANNEL_FACTORY_IFACE, _channel_factory_iface_init));
+		G_IMPLEMENT_INTERFACE(TP_TYPE_CHANNEL_FACTORY_IFACE, _factory_iface_init));
 
 /* properties */
 enum {
@@ -89,44 +87,46 @@ static void idle_muc_factory_init(IdleMUCFactory *obj) {
 }
 
 static void idle_muc_factory_get_property(GObject *object, guint property_id, GValue *value, GParamSpec *pspec) {
-  IdleMUCFactory *fac = IDLE_MUC_FACTORY(object);
-  IdleMUCFactoryPrivate *priv = IDLE_MUC_FACTORY_GET_PRIVATE(fac);
+	IdleMUCFactory *fac = IDLE_MUC_FACTORY(object);
+	IdleMUCFactoryPrivate *priv = IDLE_MUC_FACTORY_GET_PRIVATE(fac);
 
-  switch(property_id) {
-    case PROP_CONNECTION:
-      g_value_set_object(value, priv->conn);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
-      break;
-  }
+	switch (property_id) {
+		case PROP_CONNECTION:
+			g_value_set_object(value, priv->conn);
+			break;
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
+			break;
+	}
 }
 
 static void idle_muc_factory_set_property(GObject *object, guint property_id, const GValue *value, GParamSpec *pspec) {
-  IdleMUCFactory *fac = IDLE_MUC_FACTORY(object);
-  IdleMUCFactoryPrivate *priv = IDLE_MUC_FACTORY_GET_PRIVATE(fac);
+	IdleMUCFactory *fac = IDLE_MUC_FACTORY(object);
+	IdleMUCFactoryPrivate *priv = IDLE_MUC_FACTORY_GET_PRIVATE(fac);
 
-  switch(property_id) {
-    case PROP_CONNECTION:
-      priv->conn = g_value_get_object(value);
-      break;
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
-      break;
-  }
+	switch (property_id) {
+		case PROP_CONNECTION:
+			priv->conn = g_value_get_object(value);
+			break;
+
+		default:
+			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
+			break;
+	}
 }
 
 static void idle_muc_factory_class_init(IdleMUCFactoryClass *klass) {
-  GObjectClass *object_class = G_OBJECT_CLASS(klass);
-  GParamSpec *param_spec;
+	GObjectClass *object_class = G_OBJECT_CLASS(klass);
+	GParamSpec *param_spec;
 
-  g_type_class_add_private(klass, sizeof(IdleMUCFactoryPrivate));
+	g_type_class_add_private(klass, sizeof(IdleMUCFactoryPrivate));
 
-  object_class->get_property = idle_muc_factory_get_property;
-  object_class->set_property = idle_muc_factory_set_property;
+	object_class->get_property = idle_muc_factory_get_property;
+	object_class->set_property = idle_muc_factory_set_property;
 
-  param_spec = g_param_spec_object("connection", "IdleConnection object", "The IdleConnection object that owns this IM channel factory object.", IDLE_TYPE_CONNECTION, G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB);
-  g_object_class_install_property(object_class, PROP_CONNECTION, param_spec);
+	param_spec = g_param_spec_object("connection", "IdleConnection object", "The IdleConnection object that owns this IM channel factory object.", IDLE_TYPE_CONNECTION, G_PARAM_CONSTRUCT_ONLY | G_PARAM_READWRITE | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB);
+	g_object_class_install_property(object_class, PROP_CONNECTION, param_spec);
 }
 
 static IdleParserHandlerResult _numeric_error_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
@@ -145,19 +145,19 @@ static IdleParserHandlerResult _numeric_error_handler(IdleParser *parser, IdlePa
 
 	switch (code) {
 		case IDLE_PARSER_NUMERIC_BADCHANNELKEY:
-			_idle_muc_channel_badchannelkey(chan);
+			idle_muc_channel_badchannelkey(chan);
 			break;
 
 		case IDLE_PARSER_NUMERIC_BANNEDFROMCHAN:
-			_idle_muc_channel_join_error(chan, MUC_CHANNEL_JOIN_ERROR_BANNED);
+			idle_muc_channel_join_error(chan, MUC_CHANNEL_JOIN_ERROR_BANNED);
 			break;
 
 		case IDLE_PARSER_NUMERIC_CHANNELISFULL:
-			_idle_muc_channel_join_error(chan, MUC_CHANNEL_JOIN_ERROR_FULL);
+			idle_muc_channel_join_error(chan, MUC_CHANNEL_JOIN_ERROR_FULL);
 			break;
 
 		case IDLE_PARSER_NUMERIC_INVITEONLYCHAN:
-			_idle_muc_channel_join_error(chan, MUC_CHANNEL_JOIN_ERROR_INVITE_ONLY);
+			idle_muc_channel_join_error(chan, MUC_CHANNEL_JOIN_ERROR_INVITE_ONLY);
 			break;
 
 		default:
@@ -181,7 +181,7 @@ static IdleParserHandlerResult _numeric_topic_handler(IdleParser *parser, IdlePa
 	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (chan)
-		_idle_muc_channel_topic(chan, topic);
+		idle_muc_channel_topic(chan, topic);
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
 }
@@ -202,7 +202,7 @@ static IdleParserHandlerResult _numeric_topic_stamp_handler(IdleParser *parser,
 	idle_connection_emit_queued_aliases_changed(priv->conn);
 
 	if (chan)
-		_idle_muc_channel_topic_touch(chan, toucher_handle, touched);
+		idle_muc_channel_topic_touch(chan, toucher_handle, touched);
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
 }
@@ -229,7 +229,7 @@ static IdleParserHandlerResult _invite_handler(IdleParser *parser, IdleParserMes
 	if (!chan) {
 		chan = _create_channel(factory, room_handle);
 		tp_channel_factory_iface_emit_new_channel(TP_CHANNEL_FACTORY_IFACE(user_data), (TpChannelIface *) chan, NULL);
-		_idle_muc_channel_invited(chan, inviter_handle);
+		idle_muc_channel_invited(chan, inviter_handle);
 	}
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
@@ -253,7 +253,7 @@ static IdleParserHandlerResult _join_handler(IdleParser *parser, IdleParserMessa
 	if (!chan)
 		chan = _create_channel(factory, room_handle);
 
-	_idle_muc_channel_join(chan, joiner_handle);
+	idle_muc_channel_join(chan, joiner_handle);
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
 }
@@ -273,7 +273,7 @@ static IdleParserHandlerResult _kick_handler(IdleParser *parser, IdleParserMessa
 	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (chan)
-		_idle_muc_channel_kick(chan, kicked_handle, kicker_handle, message);
+		idle_muc_channel_kick(chan, kicked_handle, kicker_handle, message);
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
 }
@@ -290,7 +290,7 @@ static IdleParserHandlerResult _numeric_namereply_handler(IdleParser *parser, Id
 	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (chan)
-		_idle_muc_channel_namereply(chan, args);
+		idle_muc_channel_namereply(chan, args);
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
 }
@@ -307,7 +307,7 @@ static IdleParserHandlerResult _numeric_namereply_end_handler(IdleParser *parser
 	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (chan)
-		_idle_muc_channel_namereply_end(chan);
+		idle_muc_channel_namereply_end(chan);
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
 }
@@ -324,7 +324,7 @@ static IdleParserHandlerResult _mode_handler(IdleParser *parser, IdleParserMessa
 	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (chan)
-		_idle_muc_channel_mode(chan, args);
+		idle_muc_channel_mode(chan, args);
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
 }
@@ -338,7 +338,7 @@ static void _channel_rename_foreach(TpChannelIface *iface, gpointer user_data) {
 	IdleMUCChannel *chan = IDLE_MUC_CHANNEL(iface);
 	ChannelRenameForeachData *data = user_data;
 
-	_idle_muc_channel_rename(chan, data->old_handle, data->new_handle);
+	idle_muc_channel_rename(chan, data->old_handle, data->new_handle);
 }
 
 static IdleParserHandlerResult _nick_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
@@ -375,14 +375,13 @@ static IdleParserHandlerResult _notice_privmsg_handler(IdleParser *parser, IdleP
 		type = TP_CHANNEL_TEXT_MESSAGE_TYPE_NOTICE;
 		body = idle_ctcp_kill_blingbling(g_value_get_string(g_value_array_get_nth(args, 2)));
 	} else {
-		idle_text_decode(g_value_get_string(g_value_array_get_nth(args, 2)), &type, &body);
+		gboolean decoded = idle_text_decode(g_value_get_string(g_value_array_get_nth(args, 2)), &type, &body);
+		if (!decoded)
+			return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
 	}
 
-	if (type == -1)
-		return IDLE_PARSER_HANDLER_RESULT_NOT_HANDLED;
-
 	if (chan)
-		_idle_muc_channel_receive(chan, type, sender_handle, body);
+		idle_muc_channel_receive(chan, type, sender_handle, body);
 
 	g_free(body);
 
@@ -404,7 +403,7 @@ static IdleParserHandlerResult _part_handler(IdleParser *parser, IdleParserMessa
 	IdleMUCChannel *chan = g_hash_table_lookup(priv->channels, GUINT_TO_POINTER(room_handle));
 
 	if (chan)
-		_idle_muc_channel_part(chan, leaver_handle, message);
+		idle_muc_channel_part(chan, leaver_handle, message);
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
 }
@@ -419,7 +418,7 @@ static void _channel_quit_foreach(TpChannelIface *iface, gpointer user_data) {
 	IdleMUCChannel *chan = IDLE_MUC_CHANNEL(iface);
 	ChannelQuitForeachData *data = user_data;
 
-	_idle_muc_channel_quit(chan, data->handle, data->message);
+	idle_muc_channel_quit(chan, data->handle, data->message);
 }
 
 static IdleParserHandlerResult _quit_handler(IdleParser *parser, IdleParserMessageCode code, GValueArray *args, gpointer user_data) {
@@ -449,9 +448,9 @@ static IdleParserHandlerResult _topic_handler(IdleParser *parser, IdleParserMess
 
 	if (chan) {
 		if (topic)
-			_idle_muc_channel_topic_full(chan, setter_handle, stamp, topic);
+			idle_muc_channel_topic_full(chan, setter_handle, stamp, topic);
 		else
-			_idle_muc_channel_topic_unset(chan);
+			idle_muc_channel_topic_unset(chan);
 	}
 
 	return IDLE_PARSER_HANDLER_RESULT_HANDLED;
@@ -528,13 +527,13 @@ static TpChannelFactoryRequestStatus _iface_request(TpChannelFactoryIface *iface
 	IdleMUCFactoryPrivate *priv = IDLE_MUC_FACTORY_GET_PRIVATE(factory);
 
 	if (!g_str_equal(chan_type, TP_IFACE_CHANNEL_TYPE_TEXT))
-    return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_IMPLEMENTED;
+		return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_IMPLEMENTED;
 
-  if (handle_type != TP_HANDLE_TYPE_ROOM)
-    return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_AVAILABLE;
+	if (handle_type != TP_HANDLE_TYPE_ROOM)
+		return TP_CHANNEL_FACTORY_REQUEST_STATUS_NOT_AVAILABLE;
 
-  if (!tp_handle_is_valid(tp_base_connection_get_handles(TP_BASE_CONNECTION(priv->conn), TP_HANDLE_TYPE_ROOM), handle, error))
-    return TP_CHANNEL_FACTORY_REQUEST_STATUS_INVALID_HANDLE;
+	if (!tp_handle_is_valid(tp_base_connection_get_handles(TP_BASE_CONNECTION(priv->conn), TP_HANDLE_TYPE_ROOM), handle, error))
+		return TP_CHANNEL_FACTORY_REQUEST_STATUS_INVALID_HANDLE;
 
 	if (!priv->channels) {
 		IDLE_DEBUG("Channels hash table missing, failing request...");
@@ -545,7 +544,7 @@ static TpChannelFactoryRequestStatus _iface_request(TpChannelFactoryIface *iface
 		return TP_CHANNEL_FACTORY_REQUEST_STATUS_EXISTING;
 	} else {
 		IdleMUCChannel *chan = _create_channel(factory, handle);
-		_idle_muc_channel_join_attempt(chan);
+		idle_muc_channel_join_attempt(chan);
 
 		return TP_CHANNEL_FACTORY_REQUEST_STATUS_QUEUED;
 	}
@@ -620,8 +619,8 @@ static void _channel_join_ready_cb(IdleMUCChannel *chan, guint err, gpointer use
 		g_hash_table_remove(priv->channels, GUINT_TO_POINTER(handle));
 }
 
-static void _channel_factory_iface_init(gpointer iface, gpointer data) {
-	TpChannelFactoryIfaceClass *klass = (TpChannelFactoryIfaceClass *) iface;
+static void _factory_iface_init(gpointer g_iface, gpointer iface_data) {
+	TpChannelFactoryIfaceClass *klass = (TpChannelFactoryIfaceClass *) g_iface;
 
 	klass->close_all = _iface_close_all;
 	klass->connected = NULL;
