@@ -1,11 +1,11 @@
 /*
  * This file is part of telepathy-idle
- * 
+ *
  * Copyright (C) 2006-2007 Collabora Limited
  * Copyright (C) 2006-2007 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License 
+ * modify it under the terms of the GNU Lesser General Public License
  * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful,
@@ -59,7 +59,18 @@ struct _MessageSpec {
 	IdleParserMessageCode code;
 };
 
-const static MessageSpec message_specs[] = {
+/* Message spec key:
+ * 'I' - ignore token
+ * 'r' - token is a room name
+ * 'c' - token is a contact (nick)
+ * 'C' - token is a contact (nick) with mode characters
+ * 'v' - following token is repeated multiple times
+ * 's' - token is a string
+ * ':' - Consume all remaining tokens as a single string prefixed by ':'
+ *         (e.g. ':this is a message string')
+ * '.' - Same as ':', but optional
+ */
+static const MessageSpec message_specs[] = {
 	{"PING", "Is", IDLE_PARSER_CMD_PING},
 
 	{"INVITE", "cIcr", IDLE_PARSER_PREFIXCMD_INVITE},
@@ -94,6 +105,7 @@ const static MessageSpec message_specs[] = {
 	{"333", "IIIrcd", IDLE_PARSER_NUMERIC_TOPIC_STAMP},
 	{"305", "III", IDLE_PARSER_NUMERIC_UNAWAY},
 	{"001", "IIc", IDLE_PARSER_NUMERIC_WELCOME},
+	{"311", "IIIcssI:", IDLE_PARSER_NUMERIC_WHOISUSER},
 	{"317", "IIIcd", IDLE_PARSER_NUMERIC_WHOISIDLE},
 
 	{NULL, NULL, IDLE_PARSER_LAST_MESSAGE_CODE}
@@ -122,7 +134,7 @@ struct _IdleParserPrivate {
 	IdleConnection *conn;
 
 	/* continuation line buffer */
-	gchar split_buf[IRC_MSG_MAXLEN+3];
+	gchar split_buf[IRC_MSG_MAXLEN + 3];
 
 	/* message handlers */
 	GSList *handlers[IDLE_PARSER_LAST_MESSAGE_CODE];
@@ -186,8 +198,8 @@ static gboolean _parse_atom(IdleParser *parser, GValueArray *arr, char atom, con
 
 void idle_parser_receive(IdleParser *parser, const gchar *msg) {
 	IdleParserPrivate *priv = IDLE_PARSER_GET_PRIVATE(parser);
-	int i;
-	int lasti = 0;
+	guint i;
+	guint lasti = 0;
 	gchar *tmp;
 	gboolean line_ends = FALSE;
 	guint len;
@@ -204,7 +216,7 @@ void idle_parser_receive(IdleParser *parser, const gchar *msg) {
 					g_strlcpy(g_stpcpy(concat_buf, priv->split_buf), msg, i + 1);
 					tmp = concat_buf;
 					memset(priv->split_buf, '\0', IRC_MSG_MAXLEN + 3);
-				}	else {
+				} else {
 					tmp = g_strndup(msg + lasti, i - lasti);
 				}
 
@@ -215,9 +227,9 @@ void idle_parser_receive(IdleParser *parser, const gchar *msg) {
 					g_free(tmp);
 			}
 
-			lasti = i+1;
+			lasti = i + 1;
 			line_ends = TRUE;
-		}	else {
+		} else {
 			line_ends = FALSE;
 		}
 	}
@@ -350,7 +362,13 @@ static void _parse_and_forward_one(IdleParser *parser, gchar **tokens, IdleParse
 				iter += 2;
 			}
 		} else if ((*format == ':') || (*format == '.')) {
-			if ((iter[0][0] != ':') || (iter[0][1] == '\0')) {
+			/* Because of the way things are tokenized, if there is a space
+			 * immediately after the the ':', the current token will only be
+			 * ":", so we check that iter[1][1] is non-NULL rather than checking
+			 * iter[0][1] (since iter[0] is a NULL-terminated token string
+			 * whereas iter[1] is a pointer into the full message string
+			 */
+			if ((iter[0][0] != ':') || (iter[1][1] == '\0')) {
 				success = FALSE;
 				break;
 			}
