@@ -1,11 +1,11 @@
 /*
  * This file is part of telepathy-idle
- * 
+ *
  * Copyright (C) 2006-2007 Collabora Limited
  * Copyright (C) 2006-2007 Nokia Corporation
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Lesser General Public License 
+ * modify it under the terms of the GNU Lesser General Public License
  * version 2.1 as published by the Free Software Foundation.
  *
  * This library is distributed in the hope that it will be useful,
@@ -22,14 +22,17 @@
 #define __IDLE_TEXT_H__
 
 #include <glib.h>
+
+#define DBUS_API_SUBJECT_TO_CHANGE
 #include <dbus/dbus-glib.h>
 
 #include "idle-connection.h"
 
 G_BEGIN_DECLS
 
-void idle_text_decode(const gchar *text, TpChannelTextMessageType *type, gchar **body);
-void idle_text_send (GObject *obj, guint type, const gchar *recipient, const gchar *text, IdleConnection *conn, DBusGMethodInvocation *ctx);
+gboolean idle_text_decode(const gchar *text, TpChannelTextMessageType *type, gchar **body);
+GStrv idle_text_encode_and_split(TpChannelTextMessageType type, const gchar *recipient, const gchar *text, gsize max_msg_len, GError **error);
+void idle_text_send(GObject *obj, guint type, const gchar *recipient, const gchar *text, IdleConnection *conn, DBusGMethodInvocation *ctx);
 
 G_END_DECLS
 
